let Buffer = require('buffer').Buffer;

export class Sp8deCrypto {
    constructor(eth = undefined) {
        this.EthJS = !eth ? window.EthJS.Util : eth;
    }

    sing(privateKey, str) {
        if (!privateKey ||
            !str) {
            console.error('Invalid parameters');
            return null;
        }
        let pubKey = this.EthJS.addHexPrefix(this.EthJS.privateToAddress(privateKey).toString('hex')),
            message = `${pubKey};${str}`,
            msg = this.EthJS.hashPersonalMessage(this.EthJS.toBuffer(message)),
            signed = this.EthJS.ecsign(msg, this.EthJS.toBuffer(privateKey)),
            tx = signed.r.toString('hex') + signed.s.toString('hex') + this.EthJS.stripHexPrefix(this.EthJS.intToHex(signed.v));
        return {
            pubKey: pubKey,
            message: message,
            sig: this.EthJS.addHexPrefix(tx),
            version: '3',
            signer: 'MEW'
        };
    }

    validate(signedMessage) {
        let json, hash, sig, pubKey;
        try {
            if (!signedMessage) throw new TypeError('Please enter signed message');
            json = JSON.parse(signedMessage);
            if (json.sig.length % 2 !== 0) throw new TypeError('Invalid hex string');
            sig = Buffer.from(this.getNakedAddress(json.sig), 'hex');
            if (sig.length !== 65) throw new TypeError('Sig length is not valid');
            sig[64] = sig[64] === 0 || sig[64] === 1 ? sig[64] + 27 : sig[64];
            hash = this.EthJS.hashPersonalMessage(this.EthJS.toBuffer(json.message));
            if (json.version === '3') {
                if (json.signer === 'trezor') {
                    hash = this.getTrezorHash(json.message);
                } else if (json.signer === 'ledger') {
                    hash = this.EthJS.hashPersonalMessage(Buffer.from(json.message));
                }
            } else if (json.version === '1') {
                hash = this.EthJS.sha3(json.message);
            }
            pubKey = this.EthJS.ecrecover(
                hash,
                sig[64],
                sig.slice(0, 32),
                sig.slice(32, 64)
            );

            if (this.getNakedAddress(json.pubKey) !== this.EthJS.pubToAddress(pubKey).toString('hex')) throw new TypeError('Sig is not valid');
        } catch (e) {
            if (e instanceof SyntaxError) console.error('JSON is not valid');
            else console.error(e);
            return false;
        }
        return true;
    }

    getNakedAddress(address) {
        return address.toLowerCase().replace('0x', '');
    };

    getTrezorLenBuf(msgLen) {
        if (msgLen < 253) return Buffer.from([msgLen & 0xff]);
        else if (msgLen < 0x10000)
            return Buffer.from([253, msgLen & 0xff, (msgLen >> 8) & 0xff]);
        else {
            return Buffer.from([
                254,
                msgLen & 0xff,
                (msgLen >> 8) & 0xff,
                (msgLen >> 16) & 0xff,
                (msgLen >> 24) & 0xff
            ]);
        }
    };

    getTrezorHash(msg) {
        return this.EthJS.sha3(
            Buffer.concat([
                this.EthJS.toBuffer('\u0019Ethereum Signed Message:\n'),
                this.getTrezorLenBuf(msg.length),
                this.EthJS.toBuffer(msg)
            ])
        );
    };
}




